@@ -1,7 +1,7 @@
 /*
 
-    KFloppy
-    
+    This file is part of the KFloppy program, part of the KDE project
+
     Copyright (C) 2002 Adriaan de Groot <groot@kde.org>
     Copyright (C) 2004, 2005 Nicolas GOUTTE <goutte@kde.org>
 
@@ -16,25 +16,24 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    */
+*/
 
+#include "format.h"
 
 #include <unistd.h>
 #include <stdlib.h>
 
-#include <qtimer.h>
-#include <qregexp.h>
+#include <QTimer>
+#include <QRegExp>
 
 #include <klocale.h>
-#include <kprocess.h>
+#include <k3process.h>
 #include <kdebug.h>
 #include <kstandarddirs.h>
 
-#include "format.h"
-
-static QString extPath = QString::null;
+static QString extPath = QString();
 
 /* static */ QString findExecutable(const QString &e)
 {
@@ -50,7 +49,6 @@ static QString extPath = QString::null;
 }
 
 
-
 KFAction::KFAction(QObject *parent) :
 	QObject(parent)
 {
@@ -76,7 +74,7 @@ KFAction::~KFAction()
 class KFActionQueue_p
 {
 public:
-	QPtrList<KFAction> list;
+	QList<KFAction*> list;
 } ;
 
 KFActionQueue::KFActionQueue(QObject *parent) :
@@ -84,12 +82,13 @@ KFActionQueue::KFActionQueue(QObject *parent) :
 	d(new KFActionQueue_p)
 {
 	DEBUGSETUP;
-	d->list.setAutoDelete(true);
 }
 
 KFActionQueue::~KFActionQueue()
 {
 	DEBUGSETUP;
+	qDeleteAll(d->list);
+	d->list.clear();
 	delete d;
 }
 
@@ -98,7 +97,7 @@ void KFActionQueue::queue(KFAction *p)
 	DEBUGSETUP;
 
 	d->list.append(p);
-	DEBUGS(p->name());
+	DEBUGS(p->objectName());
 }
 
 /* virtual */ void KFActionQueue::exec()
@@ -133,7 +132,6 @@ void KFActionQueue::queue(KFAction *p)
 
 	if (!success)
 	{
-            // ### TODO: the user must be told that the action has failed.
 		DEBUGS("Action failed.");
 		emit done(this,false);
 		return;
@@ -146,7 +144,7 @@ void KFActionQueue::queue(KFAction *p)
 	}
 	else
 	{
-		kdDebug(KFAREA) << "Running action " << next->name() << endl;
+		kDebug(KFAREA) << "Running action " << next->objectName() ;
 		QObject::connect(next,SIGNAL(done(KFAction *,bool)),
 			this,SLOT(actionDone(KFAction *,bool)));
 		// Propagate signals
@@ -157,8 +155,6 @@ void KFActionQueue::queue(KFAction *p)
 }
 
 
-
-
 // Here we have names of devices. The variable
 // names are basically the linux device names,
 // replace with whatever your OS needs instead.
@@ -166,25 +162,25 @@ void KFActionQueue::queue(KFAction *p)
 //
 #ifdef ANY_LINUX
 
-const char *fd0H1440[] = { "/dev/fd0u1440", "/dev/floppy/0u1440", "/dev/fd0h1440", "/dev/fd0H1440", 0L } ;
-const char *fd0D720[] = { "/dev/fd0u720", "/dev/floppy/0u720", "/dev/fd0D720", "/dev/fd0h720", 0L };
-const char *fd0h1200[] = { "/dev/fd0h1200", "/dev/floppy/0h1200", 0L };
-const char *fd0h360[] = { "/dev/fd0u360", "/dev/floppy/0u360", "/dev/fd0h360", "/dev/fd0d360", 0L };
+const char * const fd0H1440[] = { "/dev/fd0u1440", "/dev/floppy/0u1440", "/dev/fd0h1440", "/dev/fd0H1440", 0L } ;
+const char * const fd0D720[] = { "/dev/fd0u720", "/dev/floppy/0u720", "/dev/fd0D720", "/dev/fd0h720", 0L };
+const char * const fd0h1200[] = { "/dev/fd0h1200", "/dev/floppy/0h1200", 0L };
+const char * const fd0h360[] = { "/dev/fd0u360", "/dev/floppy/0u360", "/dev/fd0h360", "/dev/fd0d360", 0L };
 
-const char *fd1H1440[] = { "/dev/fd1u1440", "/dev/floppy/1u1440","/dev/fd1h1440", "/dev/fd1H1440", 0L } ;
-const char *fd1D720[] = { "/dev/fd1u720", "/dev/floppy/1u720", "/dev/fd1D720", "/dev/fd1h720", 0L };
-const char *fd1h1200[] = { "/dev/fd1h1200", "/dev/floppy/1h1200", 0L };
-const char *fd1h360[] = { "/dev/fd1u360", "/dev/floppy/1u360","/dev/fd1h360", "/dev/fd1d360", 0L };
+const char * const fd1H1440[] = { "/dev/fd1u1440", "/dev/floppy/1u1440","/dev/fd1h1440", "/dev/fd1H1440", 0L } ;
+const char * const fd1D720[] = { "/dev/fd1u720", "/dev/floppy/1u720", "/dev/fd1D720", "/dev/fd1h720", 0L };
+const char * const fd1h1200[] = { "/dev/fd1h1200", "/dev/floppy/1h1200", 0L };
+const char * const fd1h360[] = { "/dev/fd1u360", "/dev/floppy/1u360","/dev/fd1h360", "/dev/fd1d360", 0L };
 
-const char *fd0auto[] = { "/dev/fd0", 0L };
-const char *fd1auto[] = { "/dev/fd1", 0L };
+const char * const fd0auto[] = { "/dev/fd0", 0L };
+const char * const fd1auto[] = { "/dev/fd1", 0L };
 
 #endif
 
 
 #ifdef ANY_BSD
-const char *fd0[] = { "/dev/fd0", 0L } ;
-const char *fd1[] = { "/dev/fd1", 0L } ;
+const char * const fd0[] = { "/dev/fd0", 0L } ;
+const char * const fd1[] = { "/dev/fd1", 0L } ;
 #endif
 
 // Next we have a table of device names and characteristics.
@@ -194,13 +190,13 @@ const char *fd1[] = { "/dev/fd1", 0L } ;
 //
 //
 // Note that the data originally contained in KFloppy was
-// patently false, so most of this is fake. I guess noone ever
+// patently false, so most of this is fake. I guess no one ever
 // formatted a 5.25" floppy.
 //
 // The flags field is unused in this implementation.
 //
 //
-fdinfo fdtable[] =
+const fdinfo fdtable[] =
 {
 #ifdef ANY_LINUX
         // device  drv blks trk flg
@@ -236,7 +232,6 @@ fdinfo fdtable[] =
 FloppyAction::FloppyAction(QObject *p) :
 	KFAction(p),
 	deviceInfo(0L),
-	deviceName(0L),
 	theProcess(0L)
 {
 	DEBUGSETUP;
@@ -251,27 +246,40 @@ void FloppyAction::quit()
 	KFAction::quit();
 }
 
+bool FloppyAction::configureDevice( const QString& newDeviceName )
+{
+    deviceInfo = 0; // We have not any idea what the device is
+    deviceName = newDeviceName;
+    return true; // No problem!
+}
+
 bool FloppyAction::configureDevice(int drive,int density)
 {
 	DEBUGSETUP;
 	const char *devicename = 0L;
 
 	deviceInfo=0L;
-	deviceName=0L;
+	deviceName = QString();
 
 	if ((drive<0) || (drive>1))
 	{
-		emit status(i18n("Unexpected drive number %1.").arg(drive),-1);
-		return false;
-	}
-	if (!( /* (2880==de)  || */ (1440==density) || (720==density) ||
-		(1200==density) || (360==density)))
-	{
-		emit status(i18n("Unexpected density number %1.").arg(density),-1);
+		emit status(i18n("Unexpected drive number %1.", drive),-1);
 		return false;
 	}
 
-	fdinfo *deviceinfo = fdtable;
+	const fdinfo *deviceinfo = fdtable;
+	for ( ; deviceinfo && (deviceinfo->devices) ; deviceinfo++)
+	{
+		if (deviceinfo->blocks != density)
+			continue;
+        }
+	if (!deviceinfo)
+	{
+		emit status(i18n("Unexpected density number %1.", density),-1);
+		return false;
+	}
+
+	deviceinfo = fdtable;
 	for ( ; deviceinfo && (deviceinfo->devices) ; deviceinfo++)
 	{
 		if (deviceinfo->blocks != density)
@@ -282,17 +290,17 @@ bool FloppyAction::configureDevice(int drive,int density)
 
 	if (!deviceinfo || !deviceinfo->devices)
 	{
-		emit status(i18n("Cannot find a device for drive %1 and density %2.")
-			.arg(drive).arg(density),-1);
+		emit status(i18n("Cannot find a device for drive %1 and density %2.",
+			 drive, density),-1);
 		return false;
 	}
 
-	for (const char **devices=deviceinfo->devices ;
+	for (const char* const* devices=deviceinfo->devices ;
 		*devices ; devices++)
 	{
 		if (access(*devices,W_OK)>=0)
 		{
-			kdDebug(KFAREA) << "Found device " << *devices << endl;
+			kDebug(KFAREA) << "Found device " << *devices ;
 			devicename=*devices;
 			break;
 		}
@@ -302,7 +310,7 @@ bool FloppyAction::configureDevice(int drive,int density)
 	{
 		const QString str = i18n(
 			"Cannot access %1\nMake sure that the device exists and that "
-			"you have write permission to it.").arg(deviceinfo->devices[0]);
+			"you have write permission to it.", deviceinfo->devices[0]);
 		emit status(str,-1);
 		return false;
 	}
@@ -313,7 +321,7 @@ bool FloppyAction::configureDevice(int drive,int density)
 	return true;
 }
 
-void FloppyAction::processDone(KProcess *p)
+void FloppyAction::processDone(K3Process *p)
 {
 	DEBUGSETUP;
 
@@ -325,24 +333,32 @@ void FloppyAction::processDone(KProcess *p)
 
 	if (p->normalExit())
 	{
-		emit status(QString::null,100);
-		emit done(this,!p->exitStatus());
+	        if (p->exitStatus() == 0)
+	        {
+			emit status(QString::null,100);	//krazy:exclude=nullstrassign for old broken gcc
+			emit done(this,true);
+		}
+		else
+		{
+			emit status(i18n("The program %1 terminated with an error.", theProcessName),100);
+			emit done(this,false);
+		}
 	}
 	else
 	{
-		emit status(i18n("%1 terminated abnormally.").arg(theProcessName),100);
+		emit status(i18n("The program %1 terminated abnormally.", theProcessName),100);
 		emit done(this,false);
 	}
 }
 
-void FloppyAction::processStdOut(KProcess *, char *b, int l)
+void FloppyAction::processStdOut(K3Process *, char *b, int l)
 {
 	Q_UNUSED(b);
 	Q_UNUSED(l);
-	kdDebug(KFAREA) << "stdout:" << QString::fromLatin1(b,l) << endl;
+	kDebug(KFAREA) << "stdout:" << QString::fromLatin1(b,l) ;
 }
 
-void FloppyAction::processStdErr(KProcess *p, char *b, int l)
+void FloppyAction::processStdErr(K3Process *p, char *b, int l)
 {
 	processStdOut(p,b,l);
 }
@@ -351,20 +367,20 @@ bool FloppyAction::startProcess()
 {
 	DEBUGSETUP;
 
-	connect(theProcess,SIGNAL(processExited(KProcess *)),
-		this,SLOT(processDone(KProcess *)));
-	connect(theProcess,SIGNAL(receivedStdout(KProcess *,char *,int)),
-		this,SLOT(processStdOut(KProcess *,char *,int)));
-	connect(theProcess,SIGNAL(receivedStderr(KProcess *,char *,int)),
-		this,SLOT(processStdErr(KProcess *,char *,int)));
+	connect(theProcess,SIGNAL(processExited(K3Process *)),
+		this,SLOT(processDone(K3Process *)));
+	connect(theProcess,SIGNAL(receivedStdout(K3Process *,char *,int)),
+		this,SLOT(processStdOut(K3Process *,char *,int)));
+	connect(theProcess,SIGNAL(receivedStderr(K3Process *,char *,int)),
+		this,SLOT(processStdErr(K3Process *,char *,int)));
 
         theProcess->setEnvironment( "LC_ALL", "C" ); // We need the untranslated output of the tool
-	return theProcess->start(KProcess::NotifyOnExit,
-		KProcess::AllOutput);
+	return theProcess->start(K3Process::NotifyOnExit,
+		K3Process::AllOutput);
 }
 
 
-/* static */ QString FDFormat::fdformatName = QString::null;
+/* static */ QString FDFormat::fdformatName = QString();
 
 FDFormat::FDFormat(QObject *p) :
 	FloppyAction(p),
@@ -372,7 +388,7 @@ FDFormat::FDFormat(QObject *p) :
 {
 	DEBUGSETUP;
 	theProcessName = QString::fromLatin1("fdformat");
-	setName("FDFormat");
+	setObjectName("FDFormat");
 }
 
 /* static */ bool FDFormat::runtimeCheck()
@@ -391,8 +407,9 @@ bool FDFormat::configure(bool v)
 {
 	DEBUGSETUP;
 
-	if (!deviceInfo || !deviceName)
+	if ( !deviceInfo || deviceName.isEmpty() )
 	{
+                emit status( i18n("Internal error: device not correctly defined."), -1 );
 		emit done(this,false);
 		return;
 	}
@@ -405,7 +422,7 @@ bool FDFormat::configure(bool v)
 	}
 
 	if (theProcess) delete theProcess;
-	theProcess = new KProcess;
+	theProcess = new K3Process;
 
 	formatTrackCount=0;
 
@@ -444,7 +461,7 @@ bool FDFormat::configure(bool v)
 // need, since the messages can be standardized across OSsen.
 //
 //
-void FDFormat::processStdOut(KProcess *, char *b, int l)
+void FDFormat::processStdOut(K3Process *, char *b, int l)
 {
 	DEBUGSETUP;
 	QString s;
@@ -453,12 +470,12 @@ void FDFormat::processStdOut(KProcess *, char *b, int l)
 	if (b[0]=='F')
 	{
 		formatTrackCount++;
-		emit status(QString::null,
+		emit status(QString::null,	//krazy:exclude=nullstrassign for old broken gcc
 			formatTrackCount * 100 / deviceInfo->tracks);
 	}
 	else if (b[0]=='E')
 	{
-		emit status(i18n("Error formatting track %1.").arg(formatTrackCount),-1);
+		emit status(i18n("Error formatting track %1.", formatTrackCount),-1);
 	}
 	else
 	{
@@ -471,7 +488,7 @@ void FDFormat::processStdOut(KProcess *, char *b, int l)
                             "have selected a valid floppy drive."),-1);
                     return;
 		}
-		if (s.find("/dev/")>=0)
+		if (s.indexOf("/dev/")>=0)
 		{
 			emit status(s,-1);
 			return;
@@ -482,31 +499,21 @@ void FDFormat::processStdOut(KProcess *, char *b, int l)
 	s = QString::fromLatin1(b,l);
 	DEBUGS(s);
         QRegExp regexp( "([0-9]+)" );
-        if (s.startsWith("bad data at cyl") || s.startsWith("Problem reading cylinder"))
+        if ( s.startsWith( "bad data at cyl" ) || s.contains( "Problem reading cylinder" ) )
         {
-            const int track = regexp.cap(1).toInt();
-            if (track>=0)
+            if ( regexp.indexIn( s ) > -1 )
             {
-                emit status(i18n("Low-level formatting error at track %1!").arg(track), -1);
+                const int track = regexp.cap(1).toInt();
+                emit status(i18n("Low-level formatting error at track %1.", track), -1);
             }
             else
             {
                 // This error should not happen
-                emit status(i18n("Low-level formatting error: %1").arg(s), -1);
+                emit status(i18n("Low-level formatting error: %1", s), -1);
             }
             return;
         }
-        else if ( regexp.search(s) > -1 )
-        {
-            // Normal track number (formatting or verifying)
-            const int p = regexp.cap(1).toInt();
-            if ((p>=0) && (p<deviceInfo->tracks))
-            {
-                    emit status(QString::null,
-                            p * 100 / deviceInfo->tracks);
-            }
-        }
-	else if (s.find("ioctl(FDFMTBEG)")!=-1)
+	else if (s.contains("ioctl(FDFMTBEG)"))
 	{
             emit status (i18n(
                     "Cannot access floppy or floppy drive.\n"
@@ -514,18 +521,105 @@ void FDFormat::processStdOut(KProcess *, char *b, int l)
                     "have selected a valid floppy drive."),-1);
             return;
 	}
-        else if (s.find("ioctl")!=-1)
+        else if (s.contains("busy")) // "Device or resource busy"
         {
-            emit status(i18n("Low-level format error: %1").arg(s),-1);
+            emit status(i18n("Device busy.\nPerhaps you need to unmount the floppy first."),-1);
             return;
         }
+        // Be careful to leave "iotcl" as last before checking numbers
+        else if (s.contains("ioctl"))
+        {
+            emit status(i18n("Low-level format error: %1", s),-1);
+            return;
+        }
+        // Check for numbers at last (as /dev/fd0u1440 has numbers too)
+        else if ( regexp.indexIn(s) > -1 )
+        {
+            // Normal track number (formatting or verifying)
+            const int p = regexp.cap(1).toInt();
+            if ((p>=0) && (p<deviceInfo->tracks))
+            {
+                    emit status(QString::null,	//krazy:exclude=nullstrassign for old broken gcc
+                            p * 100 / deviceInfo->tracks);
+            }
+        }
 #endif
 	return;
 }
 
 
+/* static */ QString DDZeroOut::m_ddName = QString();
 
-/* static */ QString FATFilesystem::newfs_fat = QString::null ;
+DDZeroOut::DDZeroOut(QObject *p) :
+    FloppyAction(p)
+{
+    kDebug(KFAREA) << k_funcinfo ;
+    theProcessName = QString::fromLatin1("dd");
+    setObjectName("DD");
+}
+
+/* static */ bool DDZeroOut::runtimeCheck()
+{
+    m_ddName = findExecutable("dd");
+    return (!m_ddName.isEmpty());
+}
+
+/* virtual */ void DDZeroOut::exec()
+{
+    kDebug(KFAREA) << k_funcinfo ;
+
+    if ( deviceName.isEmpty() )
+    {
+        emit status( i18n("Internal error: device not correctly defined."), -1 );
+        emit done( this, false );
+        return;
+    }
+
+    if ( m_ddName.isEmpty() )
+    {
+        emit status( i18n("Cannot find dd."), -1 );
+        emit done( this, false );
+        return;
+    }
+
+    delete theProcess;
+    theProcess = new K3Process;
+
+    *theProcess << m_ddName ;
+
+    *theProcess << "if=/dev/zero" ;
+    *theProcess << "of="+deviceName;
+
+    if ( !startProcess() )
+    {
+            emit status( i18n("Could not start dd."), -1 );
+            emit done( this, false );
+    }
+
+}
+
+void DDZeroOut::processDone(K3Process *p)
+{
+    kDebug(KFAREA) << k_funcinfo ;
+
+    if (p!=theProcess)
+    {
+            kDebug(KFAREA) << "Strange process exited." ;
+            return;
+    }
+
+    /**
+     * As we do not give a number of blocks to dd(1), it will stop
+     * with the error "No space left on device"
+     *
+     * ### TODO: really check if the exit is not on an other error and then abort the formatting
+     */
+    emit status(QString::null,100);	//krazy:exclude=nullstrassign for old broken gcc
+    emit done(this,true);
+}
+
+
+/* static */ QString FATFilesystem::newfs_fat = QString();
 
 FATFilesystem::FATFilesystem(QObject *parent) :
 	FloppyAction(parent)
@@ -533,7 +627,7 @@ FATFilesystem::FATFilesystem(QObject *parent) :
 	DEBUGSETUP;
 	runtimeCheck();
 	theProcessName=newfs_fat;
-	setName("FATFilesystem");
+	setObjectName("FATFilesystem");
 }
 
 /* static */ bool FATFilesystem::runtimeCheck()
@@ -556,9 +650,9 @@ bool FATFilesystem::configure(bool v,bool l,const QString &lbl)
 	doVerify=v;
 	doLabel=l;
 	if (l)
-		label=lbl.simplifyWhiteSpace();
+		label=lbl.simplified();
 	else
-		label=QString::null;
+		label.clear();
 
 	return true;
 }
@@ -567,8 +661,13 @@ void FATFilesystem::exec()
 {
 	DEBUGSETUP;
 
-	if (!deviceInfo || !deviceName)
+	if (
+#ifdef ANY_BSD // BSD needs the deviceInfo for the block count
+            !deviceInfo ||
+#endif
+            deviceName.isEmpty())
 	{
+                emit status( i18n("Internal error: device not correctly defined."), -1 );
 		emit done(this,false);
 		return;
 	}
@@ -581,7 +680,7 @@ void FATFilesystem::exec()
 	}
 
 	if (theProcess) delete theProcess;
-	KProcess *p = theProcess = new KProcess;
+	K3Process *p = theProcess = new K3Process;
 
 	*p << newfs_fat;
 #ifdef ANY_BSD
@@ -611,14 +710,37 @@ void FATFilesystem::exec()
 	}
 }
 
-
-
-
+void FATFilesystem::processStdOut(K3Process *, char *b, int l)
+{
+#ifdef ANY_BSD
+    // ### TODO: do some checks
+#elif defined(ANY_LINUX)
+    QString s ( QString::fromLatin1( b, l ) );
+    kDebug(KFAREA) << s ;
+    if (s.contains("mounted file system")) // "/dev/fd0 contains a mounted file system
+    {
+        emit status(i18n("Floppy is mounted.\nYou need to unmount the floppy first."),-1);
+        return;
+    }
+    else if (s.contains("busy")) // "Device or resource busy"
+    {
+        emit status(i18n("Device busy.\nPerhaps you need to unmount the floppy first."),-1);
+        return;
+    }
+# if 0
+    else if ( s.find( "mkdosfs" ) != -1 ) // DEBUG: get the program header and show it!
+    {
+        emit status( s, -1 );
+        return;
+    }
+# endif
+#endif
+}
 
 
 #ifdef ANY_BSD
 
-/* static */ QString UFSFilesystem::newfs = QString::null ;
+/* static */ QString UFSFilesystem::newfs = QString();
 
 UFSFilesystem::UFSFilesystem(QObject *parent) :
 	FloppyAction(parent)
@@ -626,7 +748,7 @@ UFSFilesystem::UFSFilesystem(QObject *parent) :
 	DEBUGSETUP;
 	runtimeCheck();
 	theProcessName=newfs;
-	setName("UFSFilesystem");
+	setObjectName("UFSFilesystem");
 }
 
 /* static */ bool UFSFilesystem::runtimeCheck()
@@ -642,38 +764,41 @@ void UFSFilesystem::exec()
 {
 	DEBUGSETUP;
 
-	if (!deviceInfo || !deviceName)
+	if ( deviceName.isEmpty() )
 	{
+                emit status( i18n("Internal error: device not correctly defined."), -1 );
 		emit done(this,false);
 		return;
 	}
 
 	if (newfs.isEmpty())
 	{
-		emit status(i18n("Cannot find a program to create UFS filesystems."),-1);
+		emit status(i18nc("BSD", "Cannot find a program to create UFS filesystems."),-1);
 		emit done(this,false);
 		return;
 	}
 
 	if (theProcess) delete theProcess;
-	KProcess *p = theProcess = new KProcess;
+	K3Process *p = theProcess = new K3Process;
 
 	*p << newfs;
-	*p << "-T" << QString("fd%1").arg(deviceInfo->blocks)
-		<< deviceName;
+
+        // ### TODO: is it still needed? (FreeBSD 5.3's man page says: "For backward compatibility.")
+        if ( deviceInfo )
+           *p << "-T" << QString("fd%1").arg(deviceInfo->blocks);
+
+        *p << deviceName;
 
 	if (!startProcess())
 	{
-		emit status(i18n("Cannot start UFS format program."),-1);
+		emit status(i18nc("BSD", "Cannot start UFS format program."),-1);
 		emit done(this,false);
 	}
 }
 #endif
 
 
-
-#ifdef ANY_LINUX
-/* static */ QString Ext2Filesystem::newfs = QString::null ;
+/* static */ QString Ext2Filesystem::newfs = QString();
 
 Ext2Filesystem::Ext2Filesystem(QObject *parent) :
 	FloppyAction(parent)
@@ -681,7 +806,7 @@ Ext2Filesystem::Ext2Filesystem(QObject *parent) :
 	DEBUGSETUP;
 	runtimeCheck();
 	theProcessName="mke2fs";
-	setName("Ext2Filesystem");
+	setObjectName("Ext2Filesystem");
 }
 
 /* static */ bool Ext2Filesystem::runtimeCheck()
@@ -699,11 +824,11 @@ bool Ext2Filesystem::configure(bool v,bool l,const QString &lbl)
 	doLabel=l;
 	if (l)
 	{
-		label=lbl.stripWhiteSpace();
+		label=lbl.trimmed();
 	}
 	else
 	{
-		label=QString::null;
+		label.clear();
 	}
 
 	return true;
@@ -713,8 +838,13 @@ void Ext2Filesystem::exec()
 {
 	DEBUGSETUP;
 
-	if (!deviceInfo || !deviceName)
+	if (
+#ifdef ANY_BSD // BSD needs the deviceInfo for the block count
+            !deviceInfo ||
+#endif
+            deviceName.isEmpty() )
 	{
+                emit status( i18n("Internal error: device not correctly defined."), -1 );
 		emit done(this,false);
 		return;
 	}
@@ -727,7 +857,7 @@ void Ext2Filesystem::exec()
 	}
 
 	if (theProcess) delete theProcess;
-	KProcess *p = theProcess = new KProcess;
+	K3Process *p = theProcess = new K3Process;
 
 	*p << newfs;
 	*p << "-q";
@@ -743,12 +873,30 @@ void Ext2Filesystem::exec()
 	}
 }
 
-
-
+void Ext2Filesystem::processStdOut(K3Process *, char *b, int l)
+{
+#ifdef ANY_BSD
+    // ### TODO: do some checks
+#elif defined(ANY_LINUX)
+    QString s ( QString::fromLatin1( b, l ) );
+    kDebug(KFAREA) << s ;
+    if (s.contains("mounted")) // "/dev/fd0 is mounted; will not make a filesystem here!"
+    {
+        emit status(i18n("Floppy is mounted.\nYou need to unmount the floppy first."),-1);
+        return;
+    }
+    else if (s.contains("busy")) // "Device or resource busy"
+    {
+        emit status(i18n("Device busy.\nPerhaps you need to unmount the floppy first."),-1);
+        return;
+    }
 #endif
+}
+
+
 
 #ifdef ANY_LINUX
-/* static */ QString MinixFilesystem::newfs = QString::null ;
+/* static */ QString MinixFilesystem::newfs = QString();
 
 MinixFilesystem::MinixFilesystem(QObject *parent) :
 	FloppyAction(parent)
@@ -756,7 +904,7 @@ MinixFilesystem::MinixFilesystem(QObject *parent) :
 	DEBUGSETUP;
 	runtimeCheck();
 	theProcessName="mkfs.minix";
-	setName("Minix2Filesystem");
+	setObjectName("Minix2Filesystem");
 }
 
 /* static */ bool MinixFilesystem::runtimeCheck()
@@ -774,11 +922,11 @@ bool MinixFilesystem::configure(bool v,bool l,const QString &lbl)
 	doLabel=l;
 	if (l)
 	{
-		label=lbl.stripWhiteSpace();
+		label=lbl.trimmed();
 	}
 	else
 	{
-		label=QString::null;
+		label.clear();
 	}
 
 	return true;
@@ -788,8 +936,9 @@ void MinixFilesystem::exec()
 {
 	DEBUGSETUP;
 
-	if (!deviceInfo || !deviceName)
+	if ( deviceName.isEmpty() )
 	{
+                emit status( i18n("Internal error: device not correctly defined."), -1 );
 		emit done(this,false);
 		return;
 	}
@@ -802,7 +951,7 @@ void MinixFilesystem::exec()
 	}
 
 	if (theProcess) delete theProcess;
-	KProcess *p = theProcess = new KProcess;
+	K3Process *p = theProcess = new K3Process;
 
 	*p << newfs;
 
@@ -818,6 +967,22 @@ void MinixFilesystem::exec()
 	}
 }
 
+void MinixFilesystem::processStdOut(K3Process *, char *b, int l)
+{
+    QString s ( QString::fromLatin1( b, l ) );
+    kDebug(KFAREA) << s ;
+    if (s.contains("mounted")) // "mkfs.minix: /dev/fd0 is mounted; will not make a filesystem here!"
+    {
+        emit status(i18n("Floppy is mounted.\nYou need to unmount the floppy first."),-1);
+        return;
+    }
+    else if (s.contains("busy")) // "Device or resource busy"
+    {
+        emit status(i18n("Device busy.\nPerhaps you need to unmount the floppy first."),-1);
+        return;
+    }
+}
+
 #endif
 
 #include "format.moc"
